// packages/babel/src/index.ts
import { types as t2 } from "@babel/core";
import generate from "@babel/generator";
import hash from "@emotion/hash";

// packages/babel/src/utils.ts
import { template } from "@babel/core";
var buildImport = template("import { %%specifier%% as %%alias%% } from %%source%%;");
var buildModuleExport = template("module.exports = %%nodes%%");
var buildNamedExport = template("export { %%ident%% }");
var buildComponentStyle = template(`
const %%className%% = %%recipeImport%%(%%styles%%);
`);
var buildComponent = template(`const %%component%% = %%styledImport%%(%%tag%%, %%className%%);`);
function invariant(cond, message) {
  if (!cond) {
    throw new Error(message);
  }
}

// packages/babel/src/visitors/styled-components.ts
import { types as t } from "@babel/core";
function styledComponentsVisitor(programPath) {
  return {
    VariableDeclaration(variablePath) {
      const declarations = variablePath.get("declarations");
      if (declarations.length < 1)
        return;
      const decl = declarations[0].get("init");
      const id = declarations[0].get("id");
      if (!decl.isCallExpression())
        return;
      if (!id.isIdentifier())
        return;
      const isImportedFromLib = decl.get("callee").referencesImport("comptime-css-solid", "styled");
      if (!isImportedFromLib) {
        return;
      }
      invariant(decl.get("arguments").length === 2, "Wrong arguments to `styled`");
      const [tag, styles] = decl.get("arguments");
      const className = variablePath.scope.generateUidIdentifier(id.node.name.toLowerCase());
      const styledIdent = programPath.scope.generateUidIdentifier("$$styled");
      const recipeIdent = programPath.scope.generateUidIdentifier("recipe");
      const stmts = [
        buildImport({
          specifier: t.identifier("$$styled"),
          alias: styledIdent,
          source: t.stringLiteral("comptime-css-solid/runtime")
        }),
        buildImport({
          specifier: t.identifier("recipe"),
          alias: recipeIdent,
          source: t.stringLiteral("comptime-css")
        }),
        buildComponentStyle({
          recipeImport: recipeIdent,
          styles: styles.node,
          className
        })
      ];
      programPath.unshiftContainer("body", stmts);
      const component = buildComponent({
        component: id.node.name,
        styledImport: styledIdent,
        tag: t.cloneNode(tag.node),
        className
      });
      variablePath.replaceWith(component);
      variablePath.scope.crawl();
    }
  };
}

// packages/babel/src/index.ts
function comptimeCssBabelPlugin(opts) {
  return {
    name: "comptime-css-babel",
    visitor: {
      Program: {
        enter(path, state) {
          state.dependentNodes = /* @__PURE__ */ new Set();
          state.styledNodes = [];
          path.traverse(styledComponentsVisitor(path), state);
        },
        exit(path, state) {
          let ast = buildModuleExport({
            nodes: t2.objectExpression(state.styledNodes.map((node) => t2.objectProperty(t2.identifier(node.ident), t2.identifier(node.ident))))
          });
          const program = t2.program([...state.dependentNodes, ast].map((path2) => {
            if (!("parent" in path2))
              return path2;
            if (path2.parent.type === "Program") {
              return path2.node;
            } else {
              return path2.parent;
            }
          }));
          let cssExtract = generate(program).code;
          let cssFile = `extracted_${hash(cssExtract)}.css.ts`;
          for (const name of state.styledNodes) {
            path.unshiftContainer("body", buildImport({
              specifier: t2.identifier(name.ident),
              alias: t2.identifier(name.ident),
              source: t2.stringLiteral(cssFile)
            }));
            if (name.shouldExport) {
              path.pushContainer("body", buildNamedExport({ ident: name.ident }));
            }
          }
          path.scope.crawl();
          opts.result = [cssFile, cssExtract];
        }
      },
      VariableDeclaration(variablePath, variableState) {
        const declarations = variablePath.get("declarations");
        if (declarations.length < 1)
          return;
        const decl = declarations[0].get("init");
        const id = declarations[0].get("id");
        if (!decl.isCallExpression())
          return;
        if (!id.isArrayPattern() && !id.isIdentifier())
          return;
        const extractionAPIs = [
          "style",
          "styleVariants",
          "globalStyle",
          "createTheme",
          "createGlobalTheme",
          "createThemeContract",
          "createGlobalThemeContract",
          "assignVars",
          "createVar",
          "fallbackVar",
          "fontFace",
          "globalFontFace",
          "keyframes",
          "globalKeyframes",
          "style",
          "styleVariants",
          "globalStyle",
          "createTheme",
          "createGlobalTheme",
          "createThemeContract",
          "createGlobalThemeContract",
          "assignVars",
          "createVar",
          "fallbackVar",
          "fontFace",
          "globalFontFace",
          "keyframes",
          "globalKeyframes",
          "recipe"
        ];
        if (!extractionAPIs.some((api) => decl.get("callee").referencesImport("comptime-css", api))) {
          return;
        }
        for (const key in decl.scope.bindings) {
          const binding = decl.scope.bindings[key];
          if (binding && key) {
            variableState.dependentNodes.add(binding.path);
          }
        }
        const shouldExport = variablePath.parentPath.isExportNamedDeclaration();
        if (id.isArrayPattern()) {
          for (const elementPath of id.get("elements")) {
            if (elementPath.isIdentifier()) {
              variableState.styledNodes.push({
                shouldExport,
                ident: elementPath.node.name
              });
            }
          }
        } else {
          variableState.styledNodes.push({
            shouldExport,
            ident: id.node.name
          });
        }
        variablePath.remove();
        variablePath.scope.crawl();
      }
    }
  };
}
export {
  comptimeCssBabelPlugin
};
//# sourceMappingURL=index.mjs.map